@@ -4,6 +4,8 @@ from itertools import chain
 import numpy as np
 import pandas as pd
 import mne
+import statsmodels.api as sm
+from scipy.signal import find_peaks, peak_prominences
 
 from alice_ml.utils import trim
 from alice_ml.preprocessing import IC
@@ -12,7 +14,8 @@ from alice_ml import data
 
 eye_move_example = np.load(pkg_resources.open_binary(data, 'eye_move_example.npy'))
 eye_blink_example = np.load(pkg_resources.open_binary(data, 'eye_blink_example.npy'))
-
+mean_pca = np.load(pkg_resources.open_binary(data, 'pca_mean.npy'))
+components_pca = np.load(pkg_resources.open_binary(data, 'pca_components.npy'))
 
 def _cross_corr(epoch_eye, epoch):
     ccov = np.correlate(epoch_eye - epoch_eye.mean(), epoch - epoch.mean(), mode='same')
@@ -20,6 +23,30 @@ def _cross_corr(epoch_eye, epoch):
 
     return ccor
 
+def __alpha_peak_from_psd(freqs, psd):
+    alphs_inds = np.where((freqs>=5)&(freqs<=15))[0]
+    change = np.linspace(-1, 1, len(freqs))
+    
+    psd = (10 * np.log10(psd * 100/2))
+    xs = sm.add_constant(1. / freqs)
+    
+    lr = sm.OLS(psd.T, xs).fit()
+    residuals = lr.get_influence().resid_studentized_internal
+    res_pk, _ = find_peaks(residuals[alphs_inds], height=1, 
+                          prominence=0.3, width=2.7, rel_height=0.5)
+    psd_pk, _ = find_peaks(psd[alphs_inds], height=0,
+                           prominence=0.15, width=0, rel_height=0.5)
+    
+    res_pl, _ = find_peaks(residuals[alphs_inds], height=1, prominence=0.15,
+                           width=0, rel_height=0.5)
+    psd_pl, _ = find_peaks(psd[alphs_inds] + 10*change[alphs_inds], 
+                           prominence=0.05, height=0, width=0, rel_height=0.5)
+    
+    def _to_freq(pk_list):
+        return [freqs[alphs_inds][p] for p in pk_list]
+    
+    return _to_freq(res_pk), _to_freq(psd_pk), _to_freq(res_pl), _to_freq(psd_pl), lr
+
 
 def compute_K(ic, mean_shift=False, thres=0.99) -> float:
     """
@@ -202,6 +229,42 @@ def compute_CIF(ic):
     raise NotImplementedError
 
 
+def compute_alpha_features(ic, average_epochs=False):
+    peaks_data = pd.DataFrame()
+    freqs, psd = ic.psd(verbose=False)
+
+    freq_thr = np.where((freqs>=1)&(freqs <= 50))[0]
+    freqs = freqs[freq_thr]
+    psd = psd[:, freq_thr] 
+
+    epochs_with_pk = 0
+    epochs_with_pl = 0
+    peaks_coords = []
+    plats_coords = []
+    
+    if average_epochs:
+        psd = psd.mean(axis=0).reshape(1, -1)
+    
+    for i in range(psd.shape[0]):
+        res_pk, psd_pk, res_pl, psd_pl, _ = __alpha_peak_from_psd(freqs, psd[i, :])
+
+        if len(res_pk) > 0 and len(psd_pk) > 0:
+            epochs_with_pk += 1
+            peaks_coords = peaks_coords + res_pk
+
+        if len(res_pl)>0 and len(psd_pl)>0:
+            epochs_with_pl += 1
+            plats_coords = plats_coords + res_pl
+            
+    epochs_with_pk = epochs_with_pk / psd.shape[0]
+    epochs_with_pl = epochs_with_pl / psd.shape[0]
+
+    mean_peak_freq = np.mean(peaks_coords) if len(peaks_coords) > 0 else None
+    mean_plat_freq = np.mean(plats_coords) if len(plats_coords) > 0 else None
+    
+    return epochs_with_pk, epochs_with_pl, mean_peak_freq, mean_plat_freq
+
+
 default_features = {'K': compute_K,
                     'MEV': compute_MEV,
                     'SAD': compute_SAD,
@@ -215,7 +278,7 @@ default_features = {'K': compute_K,
                     'AMALB':compute_AMALB}
 
 
-def build_feature_df(data, default=True, custom_features={}):
+def build_alice_features_df(data, default=True, custom_features={}):
     """
     Computes the feature matrix for the dataset of components.
 
@@ -228,7 +291,6 @@ def build_feature_df(data, default=True, custom_features={}):
     Returns:
         pd.Dataframe: The feature matrix for the dataset.
     """
-    feature_df = pd.DataFrame(index=data.keys())
     def get_iter():
         if default:
             return default_features.items()
@@ -244,10 +306,86 @@ def build_feature_df(data, default=True, custom_features={}):
         for feature_name, compute_feature in get_iter():
             row.append(compute_feature(ic))
         rows.append(row)
-    feature_df = pd.DataFrame(rows, index=idx, columns=features)
+    alice_feature_df = pd.DataFrame(rows, index=idx, columns=features)
+
+    return alice_feature_df
+
+
+def build_alpha_features_df(data, average_epochs=False):
+    """
+    Computes the feature matrix for the dataset of components.
+
+    Args:
+        data (dataset): A mapping of {ic_id: IC}. Compatible with the dataset representaion produced by load_dataset().
+        average_epochs (bool, optional): . Defaults to False.
+
+    Returns:
+        pd.Dataframe: The feature matrix for the dataset.
+    """
+
+    features = ['A_peaks', 'A_plateaus', 'A_peaks_freq', 'A_plateaus_freq']
+    idx = []
+    rows = []
+    for ic_id, ic in data.items():
+        row = []
+        idx.append(ic_id)
+        row = compute_alpha_features(ic)
+        rows.append(row)
+    alpha_feature_df = pd.DataFrame(rows, index=idx, columns=features)
+
+    return alpha_feature_df
+
+def build_pca_features_df(data):
+    """
+    Computes the feature matrix for the dataset of components.
+
+    Args:
+        data (dataset): A mapping of {ic_id: IC}. Compatible with the dataset representaion produced by load_dataset().
+        average_epochs (bool, optional): . Defaults to False.
+
+    Returns:
+        pd.Dataframe: The feature matrix for the dataset.
+    """
+    features = ['pca1', 'pca2']
+    idx = []
+    rows = []
+    channels =  ['fp1','fp2','f7','f3','fz','f4','f8','t3','c3','cz','c4','t4','t5','p3','pz','p4','t6','o1','o2']  
+    channels_ = ['fp1','fp2','f7','f3','fz','f4','f8','t7','c3','cz','c4','t8','p7','p3','pz','p4','p8','o1','o2'] # for EEG system with 64 channels
+    for ic_id, ic in data.items():
+        row = []
+        idx.append(ic_id)
+        try:
+            row = (ic.weights[channels].values - mean_pca) @ components_pca.T
+        except:
+            try:
+                row = (ic.weights[channels_].values- mean_pca) @ components_pca.T
+            except: # some channels have zero weights in IC, so 'ic.weights[channels_/channels]' gives errors 
+                ch_weights = []
+                for ch in channels:
+                    if ch in ic.weights.index:
+                        ch_weights.append(ic.weights[ch])
+                    else:
+                        ch_weights.append(0)
+                row = (np.array(ch_weights) - mean_pca) @ components_pca.T
+        rows.append(row)
+    pca_feature_df = pd.DataFrame(rows, index=idx, columns=features)
+
+    return pca_feature_df
+      
+def build_feature_df(data, average_epochs=False):
+    """
+    Computes the feature matrix for the dataset of components.
+
+    Args:
+        data (dataset): A mapping of {ic_id: IC}. Compatible with the dataset representaion produced by load_dataset().
+        average_epochs (bool, optional): . Defaults to False.
+
+    Returns:
+        pd.Dataframe: The feature matrix for the dataset.
+    """
+    feature_df = pd.concat([build_alice_features_df(data),build_alpha_features_df(data),build_pca_features_df(data)], axis=1)
 
     return feature_df
-  
 
 def get_features_from_mne(obj, ica_obj):
     ica_df = ica_obj.get_sources(obj).to_data_frame()
@@ -265,7 +403,7 @@ def get_features_from_mne(obj, ica_obj):
     
     channels_to_use = [ch.lower() for ch in ica_obj.info['ch_names']]
     
-    ic_names = [col for col in ica_df.columns if col not in ('epoch', 'time')]
+    ic_names = [col for col in ica_df.columns if col not in ('epoch', 'time', 'condition')]
 
     data = {}
 
@@ -282,4 +420,4 @@ def get_features_from_mne(obj, ica_obj):
 
         data[ic_name] = IC(ica_obj.info['sfreq'], signal=df_data, weights=df_weights)
     
-    return build_feature_df(data)
\ No newline at end of file
+    return build_alice_features_df(data)
